/*!
    \defgroup sensor Sensor
    \brief This module receives data from the computer.

     The module retrieves data about CPU, memory and network.
*/
///@{

#pragma once

#include <ObSensor.h>

#include <iostream>
#include <map>
#include <string>
#include <vector>

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Weverything"
#include <glibtop.h>
#include <glibtop/cpu.h>
#include <glibtop/loadavg.h>
#include <glibtop/mem.h>
#include <glibtop/netlist.h>
#include <glibtop/netload.h>
#pragma clang diagnostic pop

///
/// \class ObSensorSystem
///
/// \brief System Obmon sensor class
/// \author Martin Vala <mvala@cern.ch>
///

class ObSensorSystem : public ObSensor {
public:
  ObSensorSystem(std::string name = {"sys"});
  virtual ~ObSensorSystem() override;

  bool init() override;
  void process() override;
  void speed(ObSensor *s1, ObSensor *s2, unsigned int timeout = 1000) override;
  std::string json(const std::string name = {"static"}) const override;

  /// returns glibtop_cpu
  glibtop_cpu cpu() const { return _cpu; }

  /// returns glibtop_mem
  glibtop_mem mem() const { return _mem; }

  /// returns glibtop_netlist
  glibtop_netlist netlist() const { return _netlist; }

  /// returns netloads
  std::vector<glibtop_netload *> netloads() const { return _netloads; }

  struct DiskInfo; // FWD declaration, so it doesn't look *that* messy
  std::map<std::string, DiskInfo> disks() const { return diskInfos; }

  struct DiskInfo {
    uint64_t majorID = 0; ///< Storage device major ID
    uint64_t minorID = 0; ///< Storage device minor ID
    std::string devName;  ///< Storage device name

    uint64_t rCount = 0;   ///< Count of read operations
    uint64_t rMerged = 0;  ///< Count of merged read operations
    uint64_t rSectors = 0; ///< Count of sectors read
    uint64_t rTime = 0;    ///< Time of sector reading (? NEEDINFO)

    uint64_t wCount = 0;   ///< Count of write operations
    uint64_t wMerged = 0;  ///< Count of merged write operations
    uint64_t wSectors = 0; ///< Count of sectors written
    uint64_t wTime = 0;    ///< Time of sector writing (? NEEDINFO)

    uint64_t currentIOCount = 0; ///< Count of currently running IO ops
    uint64_t timeIO = 0;         ///< Time total taken by IO ops
    uint64_t weightedTimeIO = 0; ///< Weighted time taken by IO ops (? NEEDINFO)
  };

private:
  uint64_t sz_MiB_kiB_(unsigned long long mib) { return mib * 1024; }

  glibtop *_sysinfo;          ///< Sys info from glitop
  glibtop_cpu _cpu{};         ///< CPU info from glitop
  glibtop_mem _mem{};         ///< Mem info from glitop
  glibtop_loadavg _loadavg{}; ///< Load average info from glibtop
  glibtop_netlist _netlist{}; ///< NetList from glitop
  std::vector<glibtop_netload *> _netloads{}; ///< List of netload
  std::vector<std::string> _netnames{};       ///< list of network names

  std::vector<std::pair<std::string, bool>> allowedDisks{};
  std::map<std::string, DiskInfo> diskInfos{};
  std::vector<std::string> allowedNICs{};
  double maxDiskSpeed = static_cast<double>(sz_MiB_kiB_(100));
  int _resCpu{0}; ///< Cpus used by system (not for computation)
};

///@}
